﻿/*
 Copyright 1995-2010 ESRI

 All rights reserved under the copyright laws of the United States.

 You may freely redistribute and use this sample code, with or without modification.

 Disclaimer: THE SAMPLE CODE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED 
 WARRANTIES, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ESRI OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) SUSTAINED BY YOU OR A THIRD PARTY, HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ARISING IN ANY 
 WAY OUT OF THE USE OF THIS SAMPLE CODE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 SUCH DAMAGE.

 For additional information contact: Environmental Systems Research Institute, Inc.

 Attn: Contracts Dept.

 380 New York Street

 Redlands, California, U.S.A. 92373 

 Email: contracts@esri.com
*/

using ESRI.ArcGIS.CadastralUI;
using ESRI.ArcGIS.Geodatabase;
using ESRI.ArcGIS.GeoDatabaseExtensions;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Geometry;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.Display;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Editor;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace DeleteSelectedParcels
{
  public class clsDeleteEmptyPlans : ESRI.ArcGIS.Desktop.AddIns.Button
  {
    private IStepProgressor m_pStepProgressor;
    //Create a CancelTracker.
    private ITrackCancel m_pTrackCancel;
    private IProgressDialogFactory m_pProgressorDialogFact;
    private bool m_bShowProgressor=true;
    public clsDeleteEmptyPlans()
    {

    }

    protected override void OnClick()
    {
      IMouseCursor pMouseCursor = new MouseCursorClass();
      pMouseCursor.SetCursor(2);

      //get the plans
      UID pUID = new UIDClass();
      pUID.Value = "{114D685F-99B7-4B63-B09F-6D1A41A4DDC1}";
      ICadastralExtensionManager2 pCadExtMan = (ICadastralExtensionManager2)ArcMap.Application.FindExtensionByCLSID(pUID);
      ICadastralEditor pCadEd = (ICadastralEditor)ArcMap.Application.FindExtensionByCLSID(pUID);
      Marshal.ReleaseComObject(pUID);

      IEditor pEd = (IEditor)ArcMap.Application.FindExtensionByName("esri object editor");
      dlgEmptyPlansList pPlansListDialog = new dlgEmptyPlansList();

      IActiveView pActiveView = ArcMap.Document.ActiveView;
      IMap pMap = pActiveView.FocusMap;
      ICadastralFabric pCadFabric = null;
      clsFabricUtils FabricUTILS = new clsFabricUtils();

      //if we're in an edit session then grab the target fabric
      if (pEd.EditState == esriEditState.esriStateEditing)
      {
        pCadFabric = pCadEd.CadastralFabric;
      }
      if(pCadFabric==null)
      {
      //find the first fabric in the map
        if (!FabricUTILS.GetFabricFromMap(pMap, out pCadFabric))
        {
          MessageBox.Show
            ("No Parcel Fabric found in the map.\r\nPlease add a single fabric to the map, and try again.");
          return;
        }
      }
      ITable pPlansTable =null;
      if (pCadFabric != null)
        pPlansTable = pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTPlans);
      else
        return;
      IDataset pDS = (IDataset)pPlansTable;
      IWorkspace pWS = pDS.Workspace;

      bool bIsFileBasedGDB; 
      bool bIsUnVersioned;
      bool bUseNonVersionedEdit;
      if (!(FabricUTILS.SetupEditEnvironment(pWS, pCadFabric, pEd, 
        out bIsFileBasedGDB, out bIsUnVersioned, out bUseNonVersionedEdit)))
        return;

      IFIDSet pEmptyPlans = null;
      if (!FindEmptyPlans(pCadFabric, null, null, out pEmptyPlans))
        return;

      //Fill the list on the dialog
      AddEmptyPlansToList(pCadFabric, pEmptyPlans, pPlansListDialog);

      //Display the dialog
      DialogResult pDialogResult = pPlansListDialog.ShowDialog();
      if (pDialogResult != DialogResult.OK)
        return;
      IArray array = (IArray)pPlansListDialog.checkedListBox1.Tag;

      IFIDSet pPlansToDelete = new FIDSetClass();

      foreach (int checkedItemIndex in pPlansListDialog.checkedListBox1.CheckedIndices)
      {
        Int32 iPlansID = (Int32)array.get_Element(checkedItemIndex);
        if (iPlansID>-1)
          pPlansToDelete.Add(iPlansID);
      }

      if (bUseNonVersionedEdit)
      {
        FabricUTILS.DeleteRowsUnversioned(pWS, pPlansTable, pPlansToDelete, null, null);
      } 
      else
      {
        try
        {
          pEd.StartOperation();
          FabricUTILS.DeleteRowsByFIDSet(pPlansTable, pPlansToDelete, null, null);
          pEd.StopOperation("Delete Empty Plans");
        }
        catch (COMException ex)
        {
          MessageBox.Show(Convert.ToString(ex.ErrorCode));
          pEd.AbortOperation();
          if (pPlansToDelete != null)
            Marshal.ReleaseComObject(pPlansToDelete);
          if (pEmptyPlans != null)
            Marshal.ReleaseComObject(pEmptyPlans);
          return;
        }
      }

      Marshal.ReleaseComObject(pPlansToDelete);
      Marshal.ReleaseComObject(pEmptyPlans);

    }
    bool FindEmptyPlans(ICadastralFabric Fabric, IStepProgressor StepProgressor,
      ITrackCancel TrackCancel, out IFIDSet EmptyPlans)
    {
      ICursor pPlansCur = null;
      ICursor pParcelCur = null;
      IFIDSet pNonEmptyPlansFIDSet = new FIDSetClass();

      try
      {
        ITable pPlansTable = Fabric.get_CadastralTable(esriCadastralFabricTable.esriCFTPlans);
        ITable pParcelsTable = Fabric.get_CadastralTable(esriCadastralFabricTable.esriCFTParcels);

        //build a new FIDSet for all the plans found via parcels
        pParcelCur = pParcelsTable.Search(null, false);
        int iRowCount = pParcelsTable.RowCount(null);

        IRow pParcRow = pParcelCur.NextRow();
        Int32 iPlanIDX = pParcelCur.Fields.FindField("PLANID");

        IProgressDialog2 pProgressorDialog = null;

        m_pProgressorDialogFact = new ProgressDialogFactory();
        m_pTrackCancel = new CancelTrackerClass();
        m_pStepProgressor = m_pProgressorDialogFact.Create(m_pTrackCancel, ArcMap.Application.hWnd);
        pProgressorDialog = (IProgressDialog2)m_pStepProgressor;
        m_pStepProgressor.MinRange = 1;
        if (m_bShowProgressor)
        {
          m_pStepProgressor.MaxRange = iRowCount;
          m_pStepProgressor.StepValue = 1;
          pProgressorDialog.Animation = ESRI.ArcGIS.Framework.esriProgressAnimationTypes.esriProgressSpiral;
          pProgressorDialog.ShowDialog();
          m_pStepProgressor.Message = "Finding empty plans...";
          m_pStepProgressor.Step();
        }
        bool bCont = true;
        while (pParcRow != null)
        {
          //Check if the cancel button was pressed. If so, stop process   
          if (m_bShowProgressor)
          {
            bCont = m_pTrackCancel.Continue();
            if (!bCont)
              break;
          }
          //Create a collection of planIDs for plans that we know are not empty
          Int32 iPlanID = (Int32)pParcRow.get_Value(iPlanIDX);
          if (iPlanID>-1)
            pNonEmptyPlansFIDSet.Add(iPlanID);
          Marshal.ReleaseComObject(pParcRow);
          pParcRow = pParcelCur.NextRow();
          if (m_bShowProgressor)
          {
            if (m_pStepProgressor.Position < m_pStepProgressor.MaxRange)
              m_pStepProgressor.Step();
          }
        }
                
        if (pParcelCur!=null)
           Marshal.ReleaseComObject(pParcelCur);
        if (!(pProgressorDialog == null))
          pProgressorDialog.HideDialog();
        pProgressorDialog = null;

        if (!bCont)
        {
          if (pNonEmptyPlansFIDSet != null)
            Marshal.ReleaseComObject(pNonEmptyPlansFIDSet);
          EmptyPlans = null;
          return false;
        }
        
        //ISelectionSet2 pSelSet = null;
        //try
        //{
        //  IFeatureSelection pTableSel = (IFeatureSelection)pPlansTable;
        //  //Get the selection on plan table
        //  pSelSet = (ISelectionSet2)pTableSel.SelectionSet;
        //}
        //catch (COMException ex)
        //{
        //  Debug.WriteLine (Convert.ToString(ex.ErrorCode));
        //  EmptyPlans = null;
        //  return false;
        //}


        //if (pSelSet.Count == 0)
        //{
          pPlansCur = pPlansTable.Search(null, false);
        //}
        //else
        //{
        //  pSelSet.Search(null, false, out pPlansCur);
        //}

        IFIDSet pEmptyPlansFIDSet = new FIDSetClass();
        IRow pPlanRow = pPlansCur.NextRow();
        while (pPlanRow != null)
        {
          bool bFound = false;
          pNonEmptyPlansFIDSet.Find(pPlanRow.OID, out bFound);
          if (!bFound) //This plan was not found in our parcel-referenced plans
          {
            //check if this is the default map plan, so it can be excluded from deletion
            Int32 iPlanNameIDX = pPlanRow.Fields.FindField("NAME");
            string sPlanName = (string)pPlanRow.get_Value(iPlanNameIDX);
            if(sPlanName.ToUpper()!="<MAP>")
              pEmptyPlansFIDSet.Add(pPlanRow.OID);
          }
          Marshal.ReleaseComObject(pPlanRow);
          pPlanRow = pPlansCur.NextRow();
        }
        EmptyPlans = pEmptyPlansFIDSet;
        if (pPlansCur!=null)
          Marshal.ReleaseComObject(pPlansCur);
        if (pNonEmptyPlansFIDSet!=null)
          Marshal.ReleaseComObject(pNonEmptyPlansFIDSet);
        return true;
      }
      catch (COMException ex)
      {
        if (pPlansCur != null)
          Marshal.ReleaseComObject(pPlansCur);
        if (pNonEmptyPlansFIDSet != null)
          Marshal.ReleaseComObject(pNonEmptyPlansFIDSet);
        if (pParcelCur != null)
          Marshal.ReleaseComObject(pParcelCur);
        MessageBox.Show(Convert.ToString(ex.ErrorCode));
        EmptyPlans = null;
        return false;
      }
    }

    void AddEmptyPlansToList(ICadastralFabric Fabric, IFIDSet EmptyPlansFIDSet, dlgEmptyPlansList EmptyPlansList)
    {
      ITable pPlansTable = Fabric.get_CadastralTable(esriCadastralFabricTable.esriCFTPlans);
      CheckedListBox list = EmptyPlansList.checkedListBox1;
      IArray array = new ArrayClass();
      for (int idx = 0; idx <= (EmptyPlansFIDSet.Count() - 1); idx++)
      {
        // Add the name of the plan to the list
        Int32 i_x;
        Int32 iPlanName;
        iPlanName = pPlansTable.FindField("NAME");
        EmptyPlansFIDSet.Next(out i_x);
        array.Add(i_x);
        string sPlanName = (string)pPlansTable.GetRow(i_x).get_Value(iPlanName);
        list.Items.Add(sPlanName, true);
      }
      // Bind array of plan ids with the list
      list.Tag = (object)array;
      if(list.Items.Count>0)
        EmptyPlansList.checkedListBox1_SelectedValueChanged(null, null);
    }
  }
  }

