﻿/*
 Copyright 1995-2010 ESRI

 All rights reserved under the copyright laws of the United States.

 You may freely redistribute and use this sample code, with or without modification.

 Disclaimer: THE SAMPLE CODE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED 
 WARRANTIES, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ESRI OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) SUSTAINED BY YOU OR A THIRD PARTY, HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ARISING IN ANY 
 WAY OUT OF THE USE OF THIS SAMPLE CODE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 SUCH DAMAGE.

 For additional information contact: Environmental Systems Research Institute, Inc.

 Attn: Contracts Dept.

 380 New York Street

 Redlands, California, U.S.A. 92373 

 Email: contracts@esri.com
*/
using ESRI.ArcGIS.CadastralUI;
using ESRI.ArcGIS.GeoSurvey;
using ESRI.ArcGIS.Geodatabase;
using ESRI.ArcGIS.GeoDatabaseExtensions;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Geometry;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.Display;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Editor;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System;
using System.Diagnostics;

namespace DeleteSelectedParcels
{
  public class clsDeleteSelectedParcels : ESRI.ArcGIS.Desktop.AddIns.Button
  {
    private IStepProgressor m_pStepProgressor;
    //Create a CancelTracker.
    private ITrackCancel m_pTrackCancel;
    private IFIDSet m_pFIDSetLines;
    private IProgressDialogFactory m_pProgressorDialogFact;
    private IFIDSet m_pFIDSetParcels;
    private IFIDSet m_pFIDSetPoints;
    private IFIDSet m_pEmptyGeoms;
    private IQueryFilter m_pQF;
    private bool m_bShowProgressor;
    public clsDeleteSelectedParcels()
    {
 
    }

    protected override void OnClick()
    {
      IMouseCursor pMouseCursor = new MouseCursorClass();
      pMouseCursor.SetCursor(2);

      //first get the selected parcel features
      UID pUID = new UIDClass();
      pUID.Value = "{114D685F-99B7-4B63-B09F-6D1A41A4DDC1}";
      ICadastralExtensionManager2 pCadExtMan = (ICadastralExtensionManager2)ArcMap.Application.FindExtensionByCLSID(pUID);
      ICadastralEditor pCadEd = (ICadastralEditor)ArcMap.Application.FindExtensionByCLSID(pUID);
      Marshal.ReleaseComObject(pUID);
      
      IEditor pEd = (IEditor)ArcMap.Application.FindExtensionByName("esri object editor");

      IFeatureLayer CFPointLayer = null; IFeatureLayer CFLineLayer = null;
      IFeatureLayer CFControlLayer = null;
      IFeatureLayer CFLinePointLayer = null;

      IActiveView pActiveView = ArcMap.Document.ActiveView;
      IMap pMap = pActiveView.FocusMap;
      ICadastralFabric pCadFabric = null;
      clsFabricUtils FabricUTILS = new clsFabricUtils();
      IProgressDialog2 pProgressorDialog = null;

      //if we're in an edit session then grab the target fabric
      if (pEd.EditState == esriEditState.esriStateEditing)
      {  
        pCadFabric = pCadEd.CadastralFabric;
      }

      if(pCadFabric==null)
      {//find the first fabric in the map
        if (!FabricUTILS.GetFabricFromMap(pMap, out pCadFabric))
        {
          MessageBox.Show
            ("No Parcel Fabric found in the map.\r\nPlease add a single fabric to the map, and try again.");
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
          return;
        }
      }

      IArray CFParcelLayers = new ArrayClass();

      if (!(FabricUTILS.GetFabricSubLayersFromFabric(pMap, pCadFabric, out CFPointLayer, out CFLineLayer,
          out CFParcelLayers, out CFControlLayer, out CFLinePointLayer)))
        {
          return; //no fabric sublayers available for the targeted fabric
        }

     // IFeatureSelection pFeatSel = null;
      bool bIsFileBasedGDB = false; bool bIsUnVersioned = false; bool bUseNonVersionedDelete = false;
      IWorkspace pWS = null;
    try
    {
      //Get the selection of parcels
      IFeatureLayer pFL = (IFeatureLayer)CFParcelLayers.get_Element(0);
//      pFeatSel = (IFeatureSelection)pFL;
//      ISelectionSet2 pSelSet = (ISelectionSet2)pFeatSel.SelectionSet;
//      if (pSelSet.Count==0)

      IDataset pDS = (IDataset)pFL.FeatureClass;
      pWS = pDS.Workspace;

      if (!FabricUTILS.SetupEditEnvironment(pWS, pCadFabric, pEd, out bIsFileBasedGDB,
        out bIsUnVersioned, out bUseNonVersionedDelete))
      {
        Cleanup(pEd, pProgressorDialog, pMouseCursor);
        return;
      }

      if (bUseNonVersionedDelete)
      {
        ICadastralFabricLayer pCFLayer = new CadastralFabricLayerClass();
        pCFLayer.CadastralFabric = pCadFabric;
        pCadEd.CadastralFabricLayer = pCFLayer;//NOTE: Need to set this back to NULL when done.
      }      
      
      ICadastralSelection pCadaSel = (ICadastralSelection)pCadEd;

      IEnumGSParcels pEnumGSParcels = pCadaSel.SelectedParcels;// need to get the parcels before trying to get the parcel count: BUG workaround
      if(pCadaSel.SelectedParcelCount==0)
      {
        MessageBox.Show("Please select some fabric parcels and try again.","No Selection",
          MessageBoxButtons.OK,MessageBoxIcon.Information);
        pMouseCursor.SetCursor(0);
        if (bUseNonVersionedDelete)
          pCadEd.CadastralFabricLayer = null; 
        return;
      }

//     m_bShowProgressor = (pSelSet.Count > 10);
      m_bShowProgressor = (pCadaSel.SelectedParcelCount > 10);
      if (m_bShowProgressor)
      {
        m_pProgressorDialogFact = new ProgressDialogFactory();
        m_pTrackCancel = new CancelTrackerClass();
        m_pStepProgressor = m_pProgressorDialogFact.Create(m_pTrackCancel, ArcMap.Application.hWnd);
        pProgressorDialog = (IProgressDialog2)m_pStepProgressor;
        m_pStepProgressor.MinRange = 1;
 //       m_pStepProgressor.MaxRange = pSelSet.Count * 18; //(estimate 7 lines per parcel, 4 pts per parcel, 3 line points per parcel, and there is a second loop on parcel list)
        m_pStepProgressor.MaxRange = pCadaSel.SelectedParcelCount * 18; //(estimate 7 lines per parcel, 4 pts per parcel, 3 line points per parcel, and there is a second loop on parcel list)
        m_pStepProgressor.StepValue = 1;
        pProgressorDialog.Animation = ESRI.ArcGIS.Framework.esriProgressAnimationTypes.esriProgressSpiral;
      }
 //     ICursor pCursor = null;
      
      m_pQF = new QueryFilterClass();
      string sPref; string sSuff;

      ISQLSyntax pSQLSyntax = (ISQLSyntax)pWS;
      sPref = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierPrefix);
      sSuff = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierSuffix);

      
      //====== need to do this for all the parcel sublayers in the map that are part of the target fabric


//      pSelSet.Search(null, false, out pCursor);//code deletes all selected parcels
//      IFeatureCursor pParcelFeatCurs = (IFeatureCursor)pCursor;
//      IFeature pParcFeat = pParcelFeatCurs.NextFeature();
      pEnumGSParcels.Reset();
      IGSParcel pGSParcel = pEnumGSParcels.Next();
      if (m_bShowProgressor)
      {
        pProgressorDialog.ShowDialog();
        m_pStepProgressor.Message = "Collecting parcel data...";
      }

      //Add the OIDs of all the selected parcels into a new feature IDSet
      string[] sOIDList = { "(" };
      int tokenLimit = 995;
      bool bCont = true;
      int j = 0;
      int iCounter = 0;

      m_pFIDSetParcels = new FIDSetClass();

//      while (pParcFeat != null)
      while (pGSParcel != null)
      {
        //Check if the cancel button was pressed. If so, stop process   
        if (m_bShowProgressor)
        {
          bCont = m_pTrackCancel.Continue();
          if (!bCont)
            break;
        }
//        bool bExists=false;
//        m_pFIDSetParcels.Find(pParcFeat.OID,out bExists);
//        if (!bExists)
        //{
        //  m_pFIDSetParcels.Add(pParcFeat.OID);
        m_pFIDSetParcels.Add(pGSParcel.DatabaseId);
        if (iCounter <= tokenLimit)
        {
 //         sOIDList[j] = sOIDList[j] + Convert.ToString(pParcFeat.OID) + ",";
          sOIDList[j] = sOIDList[j] + Convert.ToString(pGSParcel.DatabaseId) + ",";
          iCounter++;
        }
        else
        {//maximum tokens reached
          sOIDList[j] = sOIDList[j].Trim();
          iCounter = 0;
          //set up the next OIDList
          j++;
          FabricUTILS.RedimPreserveString(ref sOIDList, 1);
          sOIDList[j] = "(";
//          sOIDList[j] = sOIDList[j] + Convert.ToString(pParcFeat.OID) + ",";
          sOIDList[j] = sOIDList[j] + Convert.ToString(pGSParcel.DatabaseId) + ",";
        }
//        Marshal.ReleaseComObject(pParcFeat); //garbage collection
//        pParcFeat = pParcelFeatCurs.NextFeature();
        Marshal.ReleaseComObject(pGSParcel); //garbage collection
        pGSParcel = pEnumGSParcels.Next();
        //}
        if (m_bShowProgressor)
        {
          if (m_pStepProgressor.Position < m_pStepProgressor.MaxRange)
            m_pStepProgressor.Step();
        }      
      }
//      Marshal.ReleaseComObject(pParcelFeatCurs); //garbage collection
      Marshal.ReleaseComObject(pEnumGSParcels); //garbage collection
//      Marshal.ReleaseComObject(pCursor); //garbage collection

      
      //=========================================================
      
      
      if (!bCont)
      {
        Cleanup(pEd, pProgressorDialog, pMouseCursor);

        //Since I'm using update cursor need to clear the cadastral selection
        pCadaSel.SelectedParcels = null;
        if (bUseNonVersionedDelete)
          pCadEd.CadastralFabricLayer = null;
       //clear selection, to make sure the parcel explorer is updated and refreshed properly
       // pFeatSel.Clear();
        RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
        return;
      }

      string sTime="";
      if (!bIsUnVersioned)
      {
        //see if parcel locks can be obtained on the selected parcels. First create a job.
        DateTime localNow = DateTime.Now;
        sTime = Convert.ToString(localNow);
        ICadastralJob pJob = new CadastralJobClass();
        pJob.Name = sTime;
        pJob.Owner = System.Windows.Forms.SystemInformation.UserName;
        pJob.Description = "Delete selected parcels";
        try
        {
          Int32 jobId = pCadFabric.CreateJob(pJob);
        }
        catch (COMException ex)
        {
          if (ex.ErrorCode == (int)fdoError.FDO_E_CADASTRAL_FABRIC_JOB_ALREADY_EXISTS)
          {
            MessageBox.Show("Job named: '" + pJob.Name + "', already exists");
          }
          else
          {
            MessageBox.Show(ex.Message);
          }
          m_pStepProgressor = null;
          if (!(pProgressorDialog == null))
            pProgressorDialog.HideDialog();
          pProgressorDialog = null;
          Marshal.ReleaseComObject(pJob);
          if (bUseNonVersionedDelete)
            pCadEd.CadastralFabricLayer = null; 
          return;
        }
        Marshal.ReleaseComObject(pJob);
      }

      //if we're in an enterprise then test for edit locks
      ICadastralFabricLocks pFabLocks = (ICadastralFabricLocks)pCadFabric;
      if (!bIsUnVersioned)
      {
        pFabLocks.LockingJob = sTime;
        ILongArray pLocksInConflict = null;
        ILongArray pSoftLcksInConflict = null;

        ILongArray pParcelsToLock = new LongArrayClass();

        FabricUTILS.FIDsetToLongArray(m_pFIDSetParcels, ref pParcelsToLock,m_pStepProgressor);
        if (m_pStepProgressor != null && !bIsFileBasedGDB)
          m_pStepProgressor.Message = "Testing for edit locks on parcels...";
        
        try
        {
          pFabLocks.AcquireLocks(pParcelsToLock, true, ref pLocksInConflict, ref pSoftLcksInConflict);
        }
        catch (COMException pCOMEx)
        {
          if (pCOMEx.ErrorCode == (int)fdoError.FDO_E_CADASTRAL_FABRIC_JOB_LOCK_ALREADY_EXISTS)
          {
            MessageBox.Show("Edit Locks could not be acquired on all selected parcels.");
            // since the operation is being aborted, release any locks that were acquired
            pFabLocks.UndoLastAcquiredLocks();
            Cleanup(pEd, pProgressorDialog, pMouseCursor);
            //clear selection, to make sure the parcel explorer is updated and refreshed properly
            //pFeatSel.Clear();
            RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
          }
          else
          {
            MessageBox.Show(Convert.ToString(pCOMEx.ErrorCode));
          }
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
          if (bUseNonVersionedDelete)
            pCadEd.CadastralFabricLayer = null;
          return;
        }
        Marshal.ReleaseComObject(pSoftLcksInConflict);
        Marshal.ReleaseComObject(pParcelsToLock);
        Marshal.ReleaseComObject(pLocksInConflict);
      }

      //Build an IDSet of lines for the parcel to be deleted, and build an IDSet of the points for those lines
      m_pFIDSetLines = new FIDSetClass();
      m_pFIDSetPoints = new FIDSetClass();
      if (pEd.EditState == esriEditState.esriStateEditing)
        pEd.StartOperation();

      if (bUseNonVersionedDelete)
      {
        if (!FabricUTILS.StartEditing(pWS))
        {
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
          if (bUseNonVersionedDelete)
            pCadEd.CadastralFabricLayer = null;
          return;
        }
      }

      //first delete all the parcel records
      if (m_pStepProgressor!=null)
        m_pStepProgressor.Message = "Deleting parcels...";

      bool bSuccess = true;
      ITable pParcelsTable = (ITable)pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTParcels);
      ITable pLinesTable = (ITable)pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTLines);
      ITable pLinePtsTable = (ITable)pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTLinePoints);
      ITable pPointsTable = (ITable)pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTPoints);
      ITable pControlTable = (ITable)pCadFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTControl);

      if (!bUseNonVersionedDelete)
        bSuccess = FabricUTILS.DeleteRowsByFIDSet(pParcelsTable, m_pFIDSetParcels,m_pStepProgressor,m_pTrackCancel);
      if (bUseNonVersionedDelete)
        bSuccess = FabricUTILS.DeleteRowsUnversioned(pWS, pParcelsTable,
            m_pFIDSetParcels, m_pStepProgressor, m_pTrackCancel);
      if (!bSuccess)
      {
        if(!bIsUnVersioned)
          pFabLocks.UndoLastAcquiredLocks();
        if (bUseNonVersionedDelete)
          FabricUTILS.AbortEditing(pWS);
        Cleanup(pEd, pProgressorDialog, pMouseCursor);
        //clear selection, to make sure the parcel explorer is updated and refreshed properly
        //pFeatSel.Clear();
        RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
        if (bUseNonVersionedDelete)
          pCadEd.CadastralFabricLayer = null;
        return;      
      }

      //next need to use an in clause for lines, so ...
      string[] sPointOIDList = {""};
      int iCnt=0;
      int iTokenCnt=0;
      int iStepCnt=1;
      //...for each item in the sOIDList array
      foreach (string inClause in sOIDList)
      {
        ICursor pLineCurs = FabricUTILS.GetCursorFromCommaSeparatedOIDList(pLinesTable, inClause, "PARCELID");
        IRow pRow=pLineCurs.NextRow();
        Int32 iFromPt = pLinesTable.Fields.FindField("FROMPOINTID");
        Int32 iToPt = pLinesTable.Fields.FindField("TOPOINTID");

        while (pRow!=null)
        {
          if (iTokenCnt >= tokenLimit)
          {
            FabricUTILS.RedimPreserveString(ref sPointOIDList, 1);
            iTokenCnt = 0;
            iCnt++;
          }
        
          m_pFIDSetLines.Add(pRow.OID);
          Int32 i =(Int32)pRow.get_Value(iFromPt);
          if (i > -1) 
          {
            bool bExists = false;
            m_pFIDSetPoints.Find(i, out bExists);
            if (!bExists)
            {
              m_pFIDSetPoints.Add(i);
              sPointOIDList[iCnt] = sPointOIDList[iCnt] + Convert.ToString(i) + ",";
              iTokenCnt++;
            }
          }
          i = (Int32)pRow.get_Value(iToPt);
          if (i > -1)
          {
            bool bExists=false;
            m_pFIDSetPoints.Find(i, out bExists);
            if (!bExists)
            {
              m_pFIDSetPoints.Add(i);
              sPointOIDList[iCnt] = sPointOIDList[iCnt] + Convert.ToString(i) + ",";
              iTokenCnt++;
            }
          }
          Marshal.ReleaseComObject(pRow); //garbage collection
          pRow = pLineCurs.NextRow();
        }
        Marshal.ReleaseComObject(pLineCurs); //garbage collection

        for (int z = 0; z<= iCnt; z++)
        {
          //remove trailing comma
          if ((sPointOIDList[z].Trim()==""))
            break;
          if ((sPointOIDList[z].Substring(sPointOIDList[z].Length - 1, 1)) == ",")
            sPointOIDList[z] = sPointOIDList[z].Substring(0, sPointOIDList[z].Length - 1);
        }

        //delete line records based on the selected parcels
        string sMessage = "Deleting lines...";
        int iSetCnt = sOIDList.GetLength(0);
        if (iSetCnt > 1)
          sMessage += "Step " + Convert.ToString(iStepCnt) + " of " + Convert.ToString(iSetCnt);
        if (m_pStepProgressor != null)
          m_pStepProgressor.Message = sMessage;
        if (!bUseNonVersionedDelete)
          bSuccess = FabricUTILS.DeleteRowsByFIDSet(pLinesTable, m_pFIDSetLines, m_pStepProgressor, m_pTrackCancel);
        if (bUseNonVersionedDelete)
          bSuccess = FabricUTILS.DeleteRowsUnversioned(pWS, pLinesTable, m_pFIDSetLines, m_pStepProgressor, m_pTrackCancel);
        if (!bSuccess)
        {
          if (!bIsUnVersioned)
            pFabLocks.UndoLastAcquiredLocks();
          if (bUseNonVersionedDelete)
            FabricUTILS.AbortEditing(pWS);
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
          //clear selection, to make sure the parcel explorer is updated and refreshed properly
          //pFeatSel.Clear();
          RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
          if (bUseNonVersionedDelete)
            pCadEd.CadastralFabricLayer = null;
          return;
        }

        //delete the line points for the deleted parcels
        //build the list of the line points that need to be deleted.
        //IFeatureClass pFeatCLLinePoints = CFLinePointLayer.FeatureClass;
        string NewInClause = "";
        //remove trailing comma
        if ((inClause.Substring(inClause.Length - 1, 1)) == ",")
          NewInClause = inClause.Substring(0, inClause.Length - 1);

        m_pQF.WhereClause = (sPref + "PARCELID" + sSuff).Trim() + " IN " + NewInClause + ")";

        ICursor pLinePointCurs = pLinePtsTable.Search(m_pQF, false);
        IRow pLinePointFeat = pLinePointCurs.NextRow();

        //Build an IDSet of linepoints for parcels to be deleted 
        IFIDSet pFIDSetLinePoints = new FIDSetClass();

        while (pLinePointFeat != null)
        {
          pFIDSetLinePoints.Add(pLinePointFeat.OID);
          Marshal.ReleaseComObject(pLinePointFeat); //garbage collection
          pLinePointFeat = pLinePointCurs.NextRow();
        }
        iSetCnt= sOIDList.GetLength(0);
        sMessage="Deleting line-points...";
        if (iSetCnt>1)
          sMessage += "Step " + Convert.ToString(iStepCnt) + " of " + Convert.ToString(iSetCnt);
        if (m_pStepProgressor != null)
          m_pStepProgressor.Message = sMessage;

        if (!bUseNonVersionedDelete)
          bSuccess = FabricUTILS.DeleteRowsByFIDSet(pLinePtsTable, 
            pFIDSetLinePoints,m_pStepProgressor,m_pTrackCancel);
        if (bUseNonVersionedDelete)
          bSuccess = FabricUTILS.DeleteRowsUnversioned(pWS, pLinePtsTable,
              pFIDSetLinePoints, m_pStepProgressor, m_pTrackCancel);
        if (!bSuccess)
        {
          if (!bIsUnVersioned)
            pFabLocks.UndoLastAcquiredLocks();
          if (bUseNonVersionedDelete)
            FabricUTILS.AbortEditing(pWS);
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
          //clear selection, to make sure the parcel explorer is updated and refreshed properly
          //pFeatSel.Clear();
          RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
          Marshal.ReleaseComObject(pFIDSetLinePoints); //garbage collection
          Marshal.ReleaseComObject(pLinePointCurs); //garbage
          if (bUseNonVersionedDelete)
            pCadEd.CadastralFabricLayer = null;
          return;
        }

        Marshal.ReleaseComObject(pFIDSetLinePoints); //garbage collection
        Marshal.ReleaseComObject(pLinePointCurs); //garbage
        iStepCnt++;
      }
      
      //now need to get points that should not be deleted, because they are used by lines that are not deleted.
      //first search for the remaining lines. Any that have from/to points that are in the point fidset define points that should stay
      IFIDSet pFIDSetNullGeomLinePtFrom = new FIDSetClass();
      IFIDSet pFIDSetNullGeomLinePtTo = new FIDSetClass();
      if (m_pStepProgressor != null)
        m_pStepProgressor.Message = "Deleting points...";
      
      for (int z = 0; z<= iCnt; z++)
      {
        if ((sPointOIDList[z].Trim() == ""))
          break;
          
          m_pQF.WhereClause = "(" + (sPref + "FROMPOINTID" + sSuff).Trim() + " IN (" + sPointOIDList[z] + ") OR " +
                               (sPref + "TOPOINTID" + sSuff).Trim() + " IN (" + sPointOIDList[z] + "))";

        IFeatureCursor pRemainingLinesCurs = (IFeatureCursor)pLinesTable.Search(m_pQF, false);
        IFeature pRow2 = pRemainingLinesCurs.NextFeature();
        
        Int32 iFromPt2 = pRemainingLinesCurs.Fields.FindField("FROMPOINTID");
        Int32 iToPt2 = pRemainingLinesCurs.Fields.FindField("TOPOINTID");

        while (pRow2!=null)
        {
          Int32 i =(Int32)pRow2.get_Value(iFromPt2);
          if (i > -1)
          {
            if (pRow2.Shape.IsEmpty)
              pFIDSetNullGeomLinePtFrom.Add(i);
            m_pFIDSetPoints.Delete(i);
            //also need to remove these from the sPointOIDList in-clause string
            string sToken = "," + Convert.ToString(i) + ",";
            if (sPointOIDList[z].Contains(sToken))
              sPointOIDList[z] = sPointOIDList[z].Replace(sToken, ",");//replace token for oid with a null string
          }
          i = (Int32)pRow2.get_Value(iToPt2);
          if (i > -1)
          {
            if (pRow2.Shape.IsEmpty)
              pFIDSetNullGeomLinePtTo.Add(i);
            m_pFIDSetPoints.Delete(i);
            //also need to remove these from the sPointOIDList in-clause string
            string sToken = "," + Convert.ToString(i) + ",";
            if (sPointOIDList[z].Contains(sToken))
              sPointOIDList[z] = sPointOIDList[z].Replace(sToken, ",");//replace token for oid with a null string       
          }
          Marshal.ReleaseComObject(pRow2); //garbage collection
          pRow2 = pRemainingLinesCurs.NextFeature();
        }
        Marshal.ReleaseComObject(pRemainingLinesCurs); //garbage collection
        //remove trailing comma
        if ((sPointOIDList[z].Substring(sPointOIDList[z].Length - 1, 1)) == ",")
          sPointOIDList[z] = sPointOIDList[z].Substring(0, sPointOIDList[z].Length - 1);
      }

      if (!bUseNonVersionedDelete)
      {
        bSuccess = FabricUTILS.DeleteRowsByFIDSet(pPointsTable, m_pFIDSetPoints,
          m_pStepProgressor, m_pTrackCancel);
      }
      if (bUseNonVersionedDelete)
        bSuccess = FabricUTILS.DeleteRowsUnversioned(pWS, pPointsTable, m_pFIDSetPoints,
          m_pStepProgressor, m_pTrackCancel);
      if (!bSuccess)
      {
        if (!bIsUnVersioned)
          pFabLocks.UndoLastAcquiredLocks();
        if (bUseNonVersionedDelete)
          FabricUTILS.AbortEditing(pWS);
        Cleanup(pEd, pProgressorDialog, pMouseCursor);
        //clear selection, to make sure the parcel explorer is updated and refreshed properly
        //pFeatSel.Clear();
        RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
        if (bUseNonVersionedDelete)
          pCadEd.CadastralFabricLayer = null;
        return;      
      }

      //Empty geometry on points that are floating points on unjoined parcels
      m_pEmptyGeoms = new FIDSetClass();
      FabricUTILS.IntersectFIDSetCommonIDs(pFIDSetNullGeomLinePtTo, pFIDSetNullGeomLinePtFrom, out m_pEmptyGeoms);
      
      ICadastralFabricSchemaEdit2 pSchemaEd = (ICadastralFabricSchemaEdit2)pCadFabric;
      pSchemaEd.ReleaseReadOnlyFields(pPointsTable, esriCadastralFabricTable.esriCFTPoints); //release safety-catch
      if (!bUseNonVersionedDelete)
        FabricUTILS.EmptyGeometries((IFeatureClass)pPointsTable, m_pEmptyGeoms);
      else
      {
        FabricUTILS.EmptyGeometriesUnversioned(pWS,CFPointLayer.FeatureClass, m_pEmptyGeoms);
      }

      pSchemaEd.ResetReadOnlyFields(esriCadastralFabricTable.esriCFTPoints);//set safety back on
      
      if (m_pStepProgressor != null)
        m_pStepProgressor.Message = "Resetting control point associations...";

      for (int z = 0; z <= iCnt; z++)
      {
        if ((sPointOIDList[z].Trim() == ""))
          break;
        //cleanup associated control points, and associations where underlying points were deleted 
        m_pQF.WhereClause = (sPref + "POINTID" + sSuff).Trim() + " IN (" + sPointOIDList[z] + ")";
        pSchemaEd.ReleaseReadOnlyFields(pControlTable, esriCadastralFabricTable.esriCFTControl); //release safety-catch
        if (!FabricUTILS.ResetControlAssociations(pControlTable, m_pQF, bUseNonVersionedDelete))
        {
          pSchemaEd.ResetReadOnlyFields(esriCadastralFabricTable.esriCFTControl);//set safety back on
          Cleanup(pEd, pProgressorDialog, pMouseCursor);
        }
        pSchemaEd.ResetReadOnlyFields(esriCadastralFabricTable.esriCFTControl);//set safety back on
      }

      Marshal.ReleaseComObject(m_pFIDSetPoints); //garbage collection    
      if (bUseNonVersionedDelete)
        FabricUTILS.StopEditing(pWS);

      if (pEd.EditState == esriEditState.esriStateEditing)
        pEd.StopOperation("Delete parcels");
      Cleanup(pEd, pProgressorDialog, pMouseCursor);

      //clear selection, to make sure the parcel explorer is updated and refreshed properly
      //pFeatSel.Clear();
      //refresh map
      RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
    }

    catch(Exception ex)
    {
      if (bUseNonVersionedDelete)
        FabricUTILS.AbortEditing(pWS);

      Cleanup(pEd, pProgressorDialog, pMouseCursor);
      //clear selection, to make sure the parcel explorer is updated and refreshed properly
//      if(pFeatSel!=null)
//        pFeatSel.Clear();
      RefreshMap(pActiveView, CFParcelLayers, CFPointLayer, CFLineLayer, CFControlLayer, CFLinePointLayer);
      if (bUseNonVersionedDelete)
        pCadEd.CadastralFabricLayer = null;
      MessageBox.Show(ex.Message);
      return;
    }
    }

    protected override void OnUpdate()
    {
      Enabled = ArcMap.Application != null;
    }

    private void RefreshMap(IActiveView ActiveView, IArray ParcelLayers, IFeatureLayer PointLayer,
      IFeatureLayer LineLayer, IFeatureLayer ControlLayer, IFeatureLayer LinePointLayer)
    {
      try
      {
        for (int z = 0; z <= ParcelLayers.Count - 1; z++)
        {
          if (ParcelLayers.get_Element(z) != null)
          {
            IFeatureSelection pFeatSel = (IFeatureSelection)ParcelLayers.get_Element(z);
            pFeatSel.Clear();//refreshes the parcel explorer
            ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeoSelection, ParcelLayers.get_Element(z), ActiveView.Extent);
            ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeography, ParcelLayers.get_Element(z), ActiveView.Extent);
          }
        }
        if (PointLayer != null)
          ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeography, PointLayer, ActiveView.Extent);
        if (LineLayer != null)
          ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeography, LineLayer, ActiveView.Extent);
        if (ControlLayer != null)
          ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeography, ControlLayer, ActiveView.Extent);
        if (LinePointLayer != null)
          ActiveView.PartialRefresh(esriViewDrawPhase.esriViewGeography, LinePointLayer, ActiveView.Extent);
      }
      catch
      {}
      }

    private void Cleanup(IEditor Editor, IProgressDialog2 ProgressorDialog, IMouseCursor MouseCursor)
    {
      MouseCursor.SetCursor(0);
      if (Editor.EditState==esriEditState.esriStateEditing)
        Editor.AbortOperation();

      if (!(ProgressorDialog == null)) 
      {
        ProgressorDialog.HideDialog(); 
      }
      ProgressorDialog = null;

      try
      {
        Marshal.ReleaseComObject(m_pQF); //garbage collection
      }
      catch{}

      try
      {
        Marshal.ReleaseComObject(m_pEmptyGeoms);
      }
      catch { }

      try
      {
        Marshal.ReleaseComObject(m_pProgressorDialogFact); //garbage collection
      }
      catch { }
      
      try
      {
        Marshal.ReleaseComObject(m_pFIDSetLines); //garbage collection
      }
      catch { }
      
      try
      {
        Marshal.ReleaseComObject(m_pFIDSetParcels); //garbage collection
      }
      catch { }
      
      try
      {
        Marshal.ReleaseComObject(m_pFIDSetPoints); //garbage collection
      }
      catch { }

      m_pStepProgressor = null;
    }
  }
}
