﻿/*
 Copyright 1995-2010 ESRI

 All rights reserved under the copyright laws of the United States.

 You may freely redistribute and use this sample code, with or without modification.

 Disclaimer: THE SAMPLE CODE IS PROVIDED "AS IS" AND ANY EXPRESS OR IMPLIED 
 WARRANTIES, INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
 FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ESRI OR 
 CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 INTERRUPTION) SUSTAINED BY YOU OR A THIRD PARTY, HOWEVER CAUSED AND ON ANY 
 THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT ARISING IN ANY 
 WAY OUT OF THE USE OF THIS SAMPLE CODE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 SUCH DAMAGE.

 For additional information contact: Environmental Systems Research Institute, Inc.

 Attn: Contracts Dept.

 380 New York Street

 Redlands, California, U.S.A. 92373 

 Email: contracts@esri.com
*/

using ESRI.ArcGIS.Geodatabase;
using ESRI.ArcGIS.GeoDatabaseExtensions;
using ESRI.ArcGIS.Carto;
using ESRI.ArcGIS.Geometry;
using ESRI.ArcGIS.Framework;
using ESRI.ArcGIS.esriSystem;
using ESRI.ArcGIS.Editor;
using ESRI.ArcGIS.CadastralUI;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System;
using System.Diagnostics;

namespace DeleteSelectedParcels
{
  class clsFabricUtils
  {
    public bool SetupEditEnvironment(IWorkspace TheWorkspace, ICadastralFabric TheFabric, IEditor TheEditor,
      out bool IsFileBasedGDB, out bool IsUnVersioned, out bool UseNonVersionedEdit)
    {
      IsFileBasedGDB = false;
      IsUnVersioned = false;
      UseNonVersionedEdit = false;

      ITable pTable = TheFabric.get_CadastralTable(esriCadastralFabricTable.esriCFTParcels);

      IsFileBasedGDB = (!(TheWorkspace.WorkspaceFactory.WorkspaceType == esriWorkspaceType.esriRemoteDatabaseWorkspace));

      if (!(IsFileBasedGDB))
      {
        IVersionedObject pVersObj = (IVersionedObject)pTable;
        IsUnVersioned = (!(pVersObj.IsRegisteredAsVersioned));
        pTable = null;
        pVersObj = null;
      }
      if (IsUnVersioned && !IsFileBasedGDB)
      {//
        DialogResult dlgRes = MessageBox.Show("Fabric is not registered as versioned." +
          "\r\n You will not be able to undo." +
          "\r\n Click 'OK' to delete permanently.",
          "Continue with delete?", MessageBoxButtons.OKCancel);
        if (dlgRes == DialogResult.OK)
        {
          UseNonVersionedEdit = true;
        }
        else if (dlgRes == DialogResult.Cancel)
        {
          return false;
        }
        //MessageBox.Show("The fabric tables are non-versioned." +
        //   "\r\n Please register as versioned, and try again.");
        //return false;
      }
      else if ((TheEditor.EditState == esriEditState.esriStateNotEditing))
      {
        MessageBox.Show("Please start editing first and try again.", "Delete",
          MessageBoxButtons.OK, MessageBoxIcon.Information);
        return false;
      }
      return true;
    }

    public void FIDsetToLongArray(IFIDSet InFIDSet, ref ILongArray OutLongArray, IStepProgressor StepProgressor)
    {
      Int32 pfID = -1;
      InFIDSet.Reset();
      double dMax = InFIDSet.Count();
      int iMax = (int)(dMax);
      for (Int32 pCnt = 0; pCnt <= (InFIDSet.Count() - 1); pCnt++)
      {
        InFIDSet.Next(out pfID);
        OutLongArray.Add(pfID);
        if (StepProgressor != null)
        {
          if (StepProgressor.Position < StepProgressor.MaxRange)
            StepProgressor.Step();
        }
      }
      return;
    }

    public void EmptyGeometries(IFeatureClass inFeatureClass, IFIDSet pFIDSet)
    {
      try
      {
        if (pFIDSet.Count() < 1)
          return;
        pFIDSet.Reset();
        int[] iID = { };
        iID = RedimPreserveInt(ref iID, pFIDSet.Count());
        for (int iCount = 0; iCount <= pFIDSet.Count() - 1; iCount++)
          pFIDSet.Next(out iID[iCount]);
        IFeatureCursor pFeatCursor = inFeatureClass.GetFeatures(iID, false);
        IFeature pFeat = pFeatCursor.NextFeature();
        while (pFeat != null)
        {
          IGeometry pGeo = pFeat.ShapeCopy;
          pGeo.SetEmpty();
          pFeat.Shape = pGeo;
          pFeat.Store();
          Marshal.ReleaseComObject(pFeat);
          pFeat = pFeatCursor.NextFeature();
        }
        Marshal.ReleaseComObject(pFeatCursor);
        iID = null;
        return;
      }
      catch (Exception ex)
      {
        MessageBox.Show(ex.Message);
        return;
      }
    }

    public void EmptyGeometriesUnversioned(IWorkspace TheWorkSpace, IFeatureClass inTable, IFIDSet pFIDSet)
    {
      IMouseCursor pMouseCursor = new MouseCursorClass();
      pMouseCursor.SetCursor(2);

      IQueryFilter pQF = new QueryFilterClass();

      ISQLSyntax pSQLSyntax = (ISQLSyntax)TheWorkSpace;
      string sPref = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierPrefix);
      string sSuff = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierSuffix);

      ICursor ipCursor = null;
      IRow pRow = null;
      //make sure that there are no more then 999 tokens for the in clause(ORA- query will otherwise error on an Oracle database)
      int iTokenLimit = 995;
      int iTokenSet = 0; //the index of the set of 995 tokens
      string sWhereClauseLHS = sPref + inTable.OIDFieldName + sSuff + " in (";
      string[] ids = { sWhereClauseLHS };

      try
      {
        ITableWrite pTableWr = (ITableWrite)inTable;
        pFIDSet.Reset();
        bool bCont = true;
        Int32 iID;

        Int32 count = pFIDSet.Count();
        int j = 0; //inner count for each set of IDs
        for (int k = 0; k < count; k++)
        {
          if (j > iTokenLimit)
          {//over the limit for this Token set, time to create a new set
            ids[iTokenSet] += ")";//close the previous set
            RedimPreserveString(ref ids, 1);//make space in the string array for the next token set
            iTokenSet++;//increment the index
            ids[iTokenSet] = sWhereClauseLHS; //left-hand side of the where clause
            j = 0;//reset the inner count back to zero
          }

          pFIDSet.Next(out iID);
          if (j > 0) //write a comma if this is not the first ID
            ids[iTokenSet] += ",";
          ids[iTokenSet] += iID.ToString();
          j++; //increment the inner count
        }
        ids[iTokenSet] += ")";

        if (count > 0)
        {
          IFeature pFeat = null;
          ISet pRowSet = new SetClass();
          for (int k = 0; k <= iTokenSet; k++)
          {
            pRowSet.RemoveAll();
            pQF.WhereClause = ids[k];
            ipCursor = pTableWr.UpdateRows(pQF, false);
            pRow = ipCursor.NextRow();
            while (pRow != null)
            {
              pFeat = (IFeature)pRow;
              IGeometry pGeo = pFeat.ShapeCopy;
              pGeo.SetEmpty();
              pFeat.Shape = pGeo;
              ipCursor.UpdateRow(pRow);
              Marshal.ReleaseComObject(pRow);
              pRow = ipCursor.NextRow();
            }
            if (!bCont)
            {
              AbortEditing(TheWorkSpace);
              if (ipCursor != null)
                Marshal.ReleaseComObject(ipCursor);
              if (pRow != null)
                Marshal.ReleaseComObject(pRow);
              if (pQF != null)
                Marshal.ReleaseComObject(pQF);
              return;
            }
          }
          Marshal.ReleaseComObject(ipCursor);
          Marshal.ReleaseComObject(pQF);
        }
        return;
      }

      catch (COMException ex)
      {
        if (ipCursor != null)
          Marshal.ReleaseComObject(ipCursor);
        if (pRow != null)
          Marshal.ReleaseComObject(pRow);
        if (pQF != null)
          Marshal.ReleaseComObject(pQF);
        MessageBox.Show(Convert.ToString(ex.ErrorCode));
        return;
      }
    }

    public void IntersectFIDSetCommonIDs(IFIDSet InFIDSet1, IFIDSet InFIDSet2, out IFIDSet OutFIDSet)
    {
      IFIDSet OutFIDSet2 = new FIDSetClass();
      Int32 pfID = -1;
      bool bExists = false;
      InFIDSet1.Reset();
      for (Int32 i = 0; i <= (InFIDSet1.Count() - 1); i++)
      {
        InFIDSet1.Next(out pfID);
        InFIDSet2.Find(pfID, out bExists);
        if (bExists)
          OutFIDSet2.Add(pfID);
      }
      OutFIDSet = OutFIDSet2;
      return;
    }

    public bool GetFabricSubLayersFromFabric(IMap Map, ICadastralFabric Fabric, out IFeatureLayer CFPointLayer, out IFeatureLayer CFLineLayer,
          out IArray CFParcelLayers, out IFeatureLayer CFControlLayer, out IFeatureLayer CFLinePointLayer)
    {
      ICadastralFabricLayer pCFLayer = null;
      ICadastralFabricSubLayer pCFSubLyr = null;
      ICompositeLayer pCompLyr = null;
      IArray CFParcelLayers2 = new ArrayClass();

      long layerCount = Map.LayerCount;
      CFPointLayer = null; CFLineLayer = null; CFControlLayer = null; CFLinePointLayer = null;
      IFeatureLayer pParcelLayer = null;
      for (int idx = 0; idx <= (layerCount - 1); idx++)
      {
        ILayer pLayer = Map.get_Layer(idx);
        bool bIsComposite = false;
        if (pLayer is ICompositeLayer)
        {
          pCompLyr = (ICompositeLayer)pLayer;
          bIsComposite = true;
        }

        int iCompositeLyrCnt = 1;
        if (bIsComposite)
          iCompositeLyrCnt = pCompLyr.Count;

        for (int i = 0; i <= (iCompositeLyrCnt - 1);i++ )
        {
          if (bIsComposite)
            pLayer=pCompLyr.get_Layer(i);
          if (pLayer is ICadastralFabricLayer)
          {
            pCFLayer = (ICadastralFabricLayer)pLayer;
            break;
          }
          if (pLayer is ICadastralFabricSubLayer)
          {  
            pCFSubLyr = (ICadastralFabricSubLayer)pLayer;
            if (pCFSubLyr.CadastralFabric.Equals(Fabric) &&
            pCFSubLyr.CadastralTableType == esriCadastralFabricTable.esriCFTParcels)
            {
              pParcelLayer = (IFeatureLayer)pCFSubLyr;
              CFParcelLayers2.Add(pParcelLayer);
              Debug.WriteLine(pParcelLayer.Name);
            }
            if (CFLineLayer == null && pCFSubLyr.CadastralFabric.Equals(Fabric) &&
            pCFSubLyr.CadastralTableType == esriCadastralFabricTable.esriCFTLines)
              CFLineLayer = (IFeatureLayer)pCFSubLyr;
            if (CFPointLayer == null && pCFSubLyr.CadastralFabric.Equals(Fabric) &&
            pCFSubLyr.CadastralTableType == esriCadastralFabricTable.esriCFTPoints)
              CFPointLayer = (IFeatureLayer)pCFSubLyr;
            if (CFLinePointLayer == null && pCFSubLyr.CadastralFabric.Equals(Fabric) &&
            pCFSubLyr.CadastralTableType == esriCadastralFabricTable.esriCFTLinePoints)
              CFLinePointLayer = (IFeatureLayer)pCFSubLyr;
            if (CFControlLayer == null && pCFSubLyr.CadastralFabric.Equals(Fabric) &&
            pCFSubLyr.CadastralTableType == esriCadastralFabricTable.esriCFTControl)
              CFControlLayer = (IFeatureLayer)pCFSubLyr;
          }
        }

        //Check that the fabric layer belongs to the requested fabric
        if (pCFLayer != null)
        {
          if (pCFLayer.CadastralFabric.Equals(Fabric))
          {
            CFPointLayer = (IFeatureLayer)pCFLayer.get_CadastralSubLayer(esriCadastralFabricRenderer.esriCFRPoints);
            CFLineLayer = (IFeatureLayer)pCFLayer.get_CadastralSubLayer(esriCadastralFabricRenderer.esriCFRLines);
            pParcelLayer = (IFeatureLayer)pCFLayer.get_CadastralSubLayer(esriCadastralFabricRenderer.esriCFRParcels);
            CFParcelLayers2.Add(pParcelLayer);
            Debug.WriteLine(pParcelLayer.Name);
            CFControlLayer = (IFeatureLayer)pCFLayer.get_CadastralSubLayer(esriCadastralFabricRenderer.esriCFRControlPoints);
            CFLinePointLayer = (IFeatureLayer)pCFLayer.get_CadastralSubLayer(esriCadastralFabricRenderer.esriCFRLinePoints);
          }
          CFParcelLayers = CFParcelLayers2;
          return true;
        }
      }
      //at the minimum, just need to make sure we have a parcel sublayer for the requested fabric
      if (pParcelLayer != null)
      {
        CFParcelLayers = CFParcelLayers2;
        return true;
      }
      else
      {
        CFParcelLayers = null;
        return false;
      }
    }

    private ICadastralFabric GetFabricFromLayer(ILayer Layer)
    { //interogates a layer and returns it's source fabric if it is a fabric layer
      ICadastralFabric Fabric = null;
      ICompositeLayer pCompLyr = null;
      ICadastralFabricSubLayer pCFSubLyr = null;
      ICadastralFabricLayer pCFLayer = null;
      bool bIsComposite = false;

      if (Layer is ICompositeLayer)
      {
        pCompLyr = (ICompositeLayer)Layer;
        bIsComposite = true;
      }

      int iCount = 1;
      if (bIsComposite)
        iCount = pCompLyr.Count;

      for (int i = 0; i <= (iCount - 1); i++)
      {
        if (bIsComposite)
          Layer = pCompLyr.get_Layer(i);
        try
        {
          pCFLayer = (ICadastralFabricLayer)Layer;
          Fabric = pCFLayer.CadastralFabric;
          return Fabric;
        }
        catch
        {
          //if it failed then try it as a fabric sublayer
          try
          {
            pCFSubLyr = (ICadastralFabricSubLayer)Layer;
            Fabric = pCFSubLyr.CadastralFabric;
            return Fabric;
          }
          catch
          {
            continue;//cast failed...not a fabric sublayer
          }
        }
      }
      return Fabric;
    }

    public bool GetFabricFromMap(IMap InMap, out ICadastralFabric Fabric)
    {//this code assumes only one fabric in the map, and will get the first that it finds.
     //Used when not in an edit session. TODO: THis could return an array of fabrics
      Fabric = null;
      for (int idx = 0; idx <= (InMap.LayerCount - 1); idx++)
      {
        ILayer pLayer = InMap.get_Layer(idx);
        Fabric = GetFabricFromLayer(pLayer);
        if (Fabric != null)
          return true;
      }
      return false;
    }

    public ICursor GetCursorFromCommaSeparatedOIDList(ITable inTable, string OIDList, string FieldName)
    {
      try
      {
        if (OIDList.Trim() == "")
          return null;
        string sPref; string sSuff;
        IDataset pDS = (IDataset)inTable;
        IWorkspace pWS = pDS.Workspace;
        ISQLSyntax pSQLSyntax = (ISQLSyntax)pWS;
        sPref = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierPrefix);
        sSuff = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierSuffix);

        //if there is no open parenthesis then add one
        if (!OIDList.Trim().StartsWith("("))
        {
          string sOIDList = ("(" + OIDList).Trim();
        }
        // remove the last comma if there is one
        if ((OIDList.Substring(OIDList.Length - 1, 1)) == ",")
          OIDList = OIDList.Substring(0, OIDList.Length - 1);
        // add a closing parenthesis if it is missing
        if (!OIDList.Trim().EndsWith(")")) OIDList = OIDList + ")";
        OIDList = OIDList.Trim();

        IQueryFilter pQueryFilter = new QueryFilter();
        pQueryFilter.WhereClause = (sPref + FieldName + sSuff).Trim() + " IN " + OIDList;
        ICursor pCursor = inTable.Search(pQueryFilter, false);
        Marshal.ReleaseComObject(pQueryFilter);//garbage collection
        return pCursor;
      }
      catch (Exception ex)
      {
        Debug.WriteLine(ex.Message);
        return null;
      }
    }

    public bool ResetControlAssociations(ITable ControlTable, IQueryFilter QueryFilter, bool Unversioned)
    {
      try
      {
        ITableWrite pTableWr = (ITableWrite)ControlTable;//used for unversioned table
        IRow pControlPointFeat = null;
        ICursor pControlPtCurs =null;

        if (Unversioned)
          pControlPtCurs = pTableWr.UpdateRows(QueryFilter, false);
        else
          pControlPtCurs = ControlTable.Update(QueryFilter, false);

        pControlPointFeat = pControlPtCurs.NextRow();

        Int32 iPointIDX = pControlPtCurs.Fields.FindField("POINTID");
        Int32 iActiveIDX = pControlPtCurs.Fields.FindField("ACTIVE");

        while (pControlPointFeat != null)
        {//loop through all of the control points, and if any of the point id values are in the deleted set, then remove the ID from the 
          //control point's Point id field
          pControlPointFeat.set_Value(iPointIDX, -1);
          pControlPointFeat.set_Value(iActiveIDX, 0);
          if (Unversioned)
            pControlPtCurs.UpdateRow(pControlPointFeat);
          else
            pControlPointFeat.Store();
          
          Marshal.ReleaseComObject(pControlPointFeat); //garbage collection
          pControlPointFeat = pControlPtCurs.NextRow();
        }
        Marshal.ReleaseComObject(pControlPtCurs); //garbage collection
        return true;
      }
      catch (COMException ex)
      {
        MessageBox.Show("Problem resetting control point association: " + Convert.ToString(ex.ErrorCode));
        return false;
      }
    }

    public bool DeleteRowsUnversioned(IWorkspace TheWorkSpace, ITable inTable,
      IFIDSet pFIDSet, IStepProgressor StepProgressor, ITrackCancel TrackCancel)
    {
      IMouseCursor pMouseCursor = new MouseCursorClass();
      pMouseCursor.SetCursor(2);

      IQueryFilter pQF = new QueryFilterClass();

      ISQLSyntax pSQLSyntax = (ISQLSyntax)TheWorkSpace;
      string sPref = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierPrefix);
      string sSuff = pSQLSyntax.GetSpecialCharacter(esriSQLSpecialCharacters.esriSQL_DelimitedIdentifierSuffix);

      ICursor ipCursor = null;
      IRow pRow = null;
      //make sure that there are no more then 999 tokens for the in clause(ORA- query will otherwise error on an Oracle database)
      int iTokenLimit = 995;
      int iTokenSet = 0; //the index of the set of 995 tokens
      string sWhereClauseLHS = sPref + inTable.OIDFieldName + sSuff + " in (";
      string[] ids = { sWhereClauseLHS };

      try
      {
        ITableWrite pTableWr = (ITableWrite)inTable;
        pFIDSet.Reset();
        bool bCont = true;
        Int32 iID;

        Int32 count = pFIDSet.Count();
        int j = 0; //inner count for each set of IDs
        for (int k = 0; k < count; k++)
        {
          if (j > iTokenLimit)
          {//over the limit for this Token set, time to create a new set
            ids[iTokenSet] += ")";//close the previous set
            RedimPreserveString(ref ids, 1);//make space in the string array for the next token set
            iTokenSet++;//increment the index
            ids[iTokenSet] = sWhereClauseLHS; //left-hand side of the where clause
            j = 0;//reset the inner count back to zero
          }
          
          pFIDSet.Next(out iID);
          if (j>0) //write a comma if this is not the first ID
            ids[iTokenSet] += ",";
          ids[iTokenSet] += iID.ToString();
          j++; //increment the inner count
        }
        ids[iTokenSet] += ")";

        if (count > 0)
        {
          for (int k = 0; k <= iTokenSet; k++)
          {
            pQF.WhereClause = ids[k];
            ipCursor = pTableWr.UpdateRows(pQF, false);
            pRow = ipCursor.NextRow();
            while (pRow != null)
            {
              ipCursor.DeleteRow();
              Marshal.ReleaseComObject(pRow);
              if (StepProgressor != null)
              {
                //Check if the cancel button was pressed. If so, stop process
                if (TrackCancel != null)
                  bCont = TrackCancel.Continue();
                if (!bCont)
                  break;
                if (StepProgressor.Position < StepProgressor.MaxRange)
                  StepProgressor.Step();
              }
              pRow = ipCursor.NextRow();
            }

            if (!bCont)
            {
              AbortEditing(TheWorkSpace);
              if (ipCursor != null)
                Marshal.ReleaseComObject(ipCursor);
              if (pRow != null)
                Marshal.ReleaseComObject(pRow);
              if (pQF != null)
                Marshal.ReleaseComObject(pQF);
              return false;
            }
          }
          Marshal.ReleaseComObject(ipCursor);
          Marshal.ReleaseComObject(pQF);
        }
        return true;
      }
    
      catch (COMException ex)
      {
        if (ipCursor!=null)
          Marshal.ReleaseComObject(ipCursor);
        if (pRow != null)
          Marshal.ReleaseComObject(pRow);
        if (pQF != null)
          Marshal.ReleaseComObject(pQF);
        MessageBox.Show(Convert.ToString(ex.ErrorCode));
        return false;
      }
    }

    public bool StartEditing(IWorkspace TheWorkspace)   // Start EditSession + create EditOperation
    {
      IWorkspaceEdit pWSEdit = (IWorkspaceEdit)TheWorkspace;
      if (pWSEdit.IsBeingEdited())
      {
        MessageBox.Show("The workspace is already being edited by another process.");
        return false;
      }
      IMultiuserWorkspaceEdit pMUWSEdit = (IMultiuserWorkspaceEdit)TheWorkspace;
      try
      {
        if (pMUWSEdit.SupportsMultiuserEditSessionMode(esriMultiuserEditSessionMode.esriMESMNonVersioned))
        {
          pMUWSEdit.StartMultiuserEditing(esriMultiuserEditSessionMode.esriMESMNonVersioned);
        }
        else
        {
          return false;
        }
      }

      catch (COMException ex)
      {
        MessageBox.Show(Convert.ToString(ex.ErrorCode));
        return false;
      }

      pWSEdit.DisableUndoRedo();
      try
      {
        pWSEdit.StartEditOperation();
      }
      catch
      {
        pWSEdit.StopEditing(false);
        return false;
      }
      return true;
    }

    public bool StopEditing(IWorkspace TheWorkspace)
    {
      IWorkspaceEdit pWSEdit = (IWorkspaceEdit)TheWorkspace;
      pWSEdit.StopEditOperation();
      pWSEdit.EnableUndoRedo();
      pWSEdit.StopEditing(true);
      return true;
    }

    public bool AbortEditing(IWorkspace TheWorkspace)
    {
      IWorkspaceEdit pWSEdit = (IWorkspaceEdit)TheWorkspace;
      pWSEdit.AbortEditOperation();
      pWSEdit.EnableUndoRedo();
      if (pWSEdit.IsBeingEdited())
        pWSEdit.StopEditing(false);
      return true;
    }

    public bool DeleteRowsByFIDSet(ITable inTable, IFIDSet pFIDSet,
      IStepProgressor StepProgressor, ITrackCancel TrackCancel)
    {//this routine uses the GetRows method, avoids the need to break up the InClause.
      IMouseCursor pMouseCursor = new MouseCursorClass();
      pMouseCursor.SetCursor(2);
      try
      {
        pFIDSet.Reset();
        int[] iID = { };
        bool bCont = true;
        iID = RedimPreserveInt(ref iID, pFIDSet.Count());
        for (int iCount = 0; iCount <= pFIDSet.Count() - 1; iCount++)
          pFIDSet.Next(out iID[iCount]);
        ICursor pCursor = inTable.GetRows(iID, false);
        IRow row = pCursor.NextRow();
        if (StepProgressor != null)
        {
          if (StepProgressor.Position < StepProgressor.MaxRange)
            StepProgressor.Step();
        }
        while (row != null)
        {
          //Check if the cancel button was pressed. If so, stop process
          if (StepProgressor != null)
          {
            if (TrackCancel != null)
              bCont = TrackCancel.Continue();
            if (!bCont)
              break;
          }
          row.Delete();
          Marshal.ReleaseComObject(row);
          row = pCursor.NextRow();
          if (StepProgressor != null)
          {
            if (StepProgressor.Position < StepProgressor.MaxRange)
              StepProgressor.Step();
          }
        }
        Marshal.ReleaseComObject(pCursor);
        inTable = null;
        iID = null;
        if (!bCont)
          return false;
        return true;
      }
      catch (COMException ex)
      {
        StepProgressor = null;
        MessageBox.Show(ex.Message);
        return false;
      }
    }

    public string[] RedimPreserveString(ref string[] x, int ResizeIncrement)
    {
      string[] Temp1 = new string[x.GetLength(0) + ResizeIncrement];
      if (x != null)
        System.Array.Copy(x, Temp1, x.Length);
      x = Temp1;
      Temp1 = null;
      return x;
    }

    public int[] RedimPreserveInt(ref int[] x, int ResizeIncrement)
    {
      int[] Temp1 = new int[x.GetLength(0) + ResizeIncrement];
      if (x != null)
        System.Array.Copy(x, Temp1, x.Length);
      x = Temp1;
      Temp1 = null;
      return x;
    }

    public long[] RedimPreserveLng(ref long[] x, long ResizeIncrement)
    {
      long[] Temp1 = new long[x.GetLength(0) + ResizeIncrement];
      if (x != null)
        System.Array.Copy(x, Temp1, x.Length);
      x = Temp1;
      Temp1 = null;
      return x;
    }

    public IFeatureLayer[] RedimPreserveFLyr(ref IFeatureLayer[] x, long ResizeIncrement)
    {
      IFeatureLayer[] Temp1 = new FeatureLayerClass[x.GetLength(0) + ResizeIncrement];
      if (x != null)
        System.Array.Copy(x, Temp1, x.Length);
      x = Temp1;
      Temp1 = null;
      return x;
    }


    public bool GetJobAndLocks(ICadastralFabric Fabric, IFIDSet FIDSetParcels, IStepProgressor StepProgressor)
    {
      string sTime = "";
      bool bIsUnVersioned = false;
      if (!bIsUnVersioned)
      {
        //see if parcel locks can be obtained on the selected parcels. First create a job.
        DateTime localNow = DateTime.Now;
        sTime = Convert.ToString(localNow);
        ICadastralJob pJob = new CadastralJobClass();
        pJob.Name = sTime;
        pJob.Owner = System.Windows.Forms.SystemInformation.UserName;
        pJob.Description = "Delete selected parcels";
        try
        {
          Int32 jobId = Fabric.CreateJob(pJob);
        }
        catch (COMException ex)
        {
          if (ex.ErrorCode == (int)fdoError.FDO_E_CADASTRAL_FABRIC_JOB_ALREADY_EXISTS)
          {
            MessageBox.Show("Job named: '" + pJob.Name + "', already exists");
          }
          else
          {
            MessageBox.Show(ex.Message);
          }
          Marshal.ReleaseComObject(pJob);
          return false;
        }
        Marshal.ReleaseComObject(pJob);
      }

      //if we're in an enterprise then test for edit locks
      ICadastralFabricLocks pFabLocks = (ICadastralFabricLocks)Fabric;
      if (!bIsUnVersioned)
      {
        pFabLocks.LockingJob = sTime;
        ILongArray pLocksInConflict = null;
        ILongArray pSoftLcksInConflict = null;

        ILongArray pParcelsToLock = new LongArrayClass();

        FIDsetToLongArray(FIDSetParcels, ref pParcelsToLock, StepProgressor);
        if (StepProgressor != null)
          StepProgressor.Message = "Testing for edit locks on parcels...";

        try
        {
          pFabLocks.AcquireLocks(pParcelsToLock, true, ref pLocksInConflict, ref pSoftLcksInConflict);
        }
        catch (COMException pCOMEx)
        {
          if (pCOMEx.ErrorCode == (int)fdoError.FDO_E_CADASTRAL_FABRIC_JOB_LOCK_ALREADY_EXISTS)
          {
            MessageBox.Show("Edit Locks could not be acquired on all selected parcels.");
            // since the operation is being aborted, release any locks that were acquired
            pFabLocks.UndoLastAcquiredLocks();
          }
          else
          {
            MessageBox.Show(Convert.ToString(pCOMEx.ErrorCode));
          }
          return false;
        }

        Marshal.ReleaseComObject(pSoftLcksInConflict);
        Marshal.ReleaseComObject(pParcelsToLock);
        Marshal.ReleaseComObject(pLocksInConflict);
        return true;
      }
      return false;
    }
  }
  }

